#include <Adafruit_GFX.h>    // Core graphics libraryplot
#include "Adafruit_ILI9341.h"
#include "MAX30100_PulseOximeter.h"
#include <OneWire.h>
#include <DallasTemperature.h>
#include <EEPROM.h>

PulseOximeter pox;

#define ONE_WIRE_BUS 6
 
// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);
 
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);

// Assign human-readable names to some common 16-bit color values:
#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define DARK_RED 0xA020
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

#define TFT_DC 9
#define TFT_CS 10

Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC,-1);

#define SCALE_HEIGHT 200            // Height of gauge on chart
#define SCALE_WIDTH 6               // Width of scale on chart
#define SCALE_Y 4                  // Y Pos start of scale
#define GAUGE_LINE_WIDTH 3
#define RESET_TEMPERATURE 43        // if any of the two sensors are above this temp then the system will go into setup screen



// globals
bool TemperatureWorking=true;
bool PulseWorking=true;
float TempReading=0;
float BloodO2Reading=0;
float HeartReading=0;
float LungReading=0;
bool WaitingForTempReading=false;
uint32_t PulseStartTime=0;            // When a beat is detected we store the start time so we can time when to turn the light off 
uint32_t RespireStartTime=0;          // When a beat is detected we store the start time so we can time when to turn the light off 
uint32_t RespireDelayStartTime=0;     // When a beat is detected we store the start time so we can time when to turn the light off 
uint16_t RespireDelay=0;              // a random anount of time before respire light flashes
bool QuickBoot=false;
bool Sound=true;

// Pointer graphic
static const unsigned char PROGMEM PointerGfx []=
{
  B00000000,B00000000,
  B00000000,B00001000,
  B00000000,B00111100,
  B00000000,B11111110,
  B00000011,B11111111,
  B00001111,B11111111,
  B00000011,B11111111,
  B00000000,B11111110,
  B00000000,B00111100,
  B00000000,B00001000,
  B00000000,B00000000,
};

// defintion struct for a gauge
struct Gauge_Struct
{
  uint16_t XPos;
  uint8_t PointerOffset;
  char Label[6];
  char SubLabel[5];
  uint16_t StartValue;
  uint16_t MaxValue;          
  uint16_t IncrementValue;
  float AmberStart;
  float AmberEnd;
  float GreenStart;
  float GreenEnd;
  float Reading;
  uint8_t Divisions;
  bool DrawFullHorizontalLine;
  float AmountToMoveBy;
  
};

Gauge_Struct Temp;
Gauge_Struct BloodO2;
Gauge_Struct Heart;
Gauge_Struct Lungs;



DeviceAddress BodyTempSensor;         // Unique ID of Temp. sensor used for the body measurements
DeviceAddress EnvironmentTempSensor;  // Unique ID of Temp. sensor used for the surroundings

void setup() {  
  tft.begin();
  tft.setRotation(1);
  pinMode(3,INPUT_PULLUP);          // button to control options, press and hold on boot to activate settings menu
  pinMode(4,OUTPUT);         // pull low to play heart beat sound
  digitalWrite(4,HIGH);
  LoadSettings();
  if(digitalRead(3)==false)
    Settings();
  BootSensors();
  InitGauges();
  DrawPanel(); 
}


void loop() {  
  if(WaitingForTempReading==false)
    UpdatePox();
  UpdateTemp();
}


void Settings()
{  
  uint8_t YPos=66;
  uint8_t Cursor=0,Choice=0;
  
  ClearScreen();
  tft.print(F("Release button."));
  // wait for button release  
  while(digitalRead(3)==false);
  DrawSettingsMenu(0,YPos);
  while(Choice!=3)
  {    
    if(digitalRead(3)==false)
    {
      delay(350);   // debounce delay 
      if(digitalRead(3)==true)     // short press
      {
        // rub out previous cursor
        tft.fillRect(300,YPos+(Cursor*16)+4, 16, 11,BLACK);
        Cursor++;
        if(Cursor==4) 
          Cursor=0;          
        tft.drawBitmap(300, YPos+(Cursor*16)+4,  PointerGfx, 16, 11,WHITE,BLACK);
      }
      else
      {
        delay(500);   // only a long press if still down after 1 sec
        if(digitalRead(3)==false)
        {
          // Long press
          switch (Cursor)
          {
            case 0:
              QuickBoot=!QuickBoot;              
              EEPROM.update(8,QuickBoot);
              BootChoice(YPos);              
              break;
            case 1:            
              Sound=!Sound;              
              EEPROM.update(9,Sound);
              HearBeatChoice(YPos+16);
              break;
            case 2:
              SetUpTempSensors(false);
              DrawSettingsMenu(Cursor,YPos);
              break;
            case 3:
              Choice=3;
              break;
          }
        }
      }
    }
  } 
}

void BootChoice(uint8_t YPos)
{
  ToggleChoice(F("1"),YPos,F("Quick Boot    "),F("Starfleet boot"),QuickBoot);
}


void HearBeatChoice(uint8_t YPos)
{
  ToggleChoice(F("2"),YPos,F("Heart Beat Sound On "),F("Heart Beat Sound Off"),Sound);  
}


void PressToMoveText()
{
  tft.setTextColor(YELLOW);
  tft.print(F("Press to move.\nLong press = change/choose."));
}

void ClearScreen()
{
  tft.fillScreen(BLACK);
  tft.setCursor(0,0);
  tft.setTextSize(2);
}


void LoadSettings()
{
  QuickBoot=EEPROM.read(8); 
  Sound=EEPROM.read(9);  
}


void DrawSettingsMenu(uint8_t Cursor,uint8_t YPos)
{
  ClearScreen();
  tft.setTextColor(GREEN);
  tft.setCursor(76,0);
  tft.println(F("   Settings"));
  tft.setTextColor(YELLOW);
  PressToMoveText();
  tft.setTextColor(WHITE,BLACK);
  BootChoice(YPos);              
  HearBeatChoice(YPos+16);
  tft.setCursor(0,YPos+32);
  tft.print(F("3. Temp Sensors"));
  tft.setCursor(0,YPos+48);
  tft.print(F("4. Exit."));
  tft.drawBitmap(300, YPos+(Cursor*16)+4,  PointerGfx, 16, 11,WHITE,BLACK);
}

void ToggleChoice(const __FlashStringHelper *IdxNo,uint8_t YPos,const __FlashStringHelper *Choice1,const __FlashStringHelper *Choice2, bool TheBool)
{   
  tft.setCursor(0,YPos);
  tft.print(IdxNo);
  tft.print(F(". "));
  if(TheBool)
    tft.print(Choice1);
  else
    tft.print(Choice2);
  while(digitalRead(3)==false);  // wait for user to release
  delay(100);  //debounce
}

void SetUpTempSensors(bool Reboot)
{
  // Allows user to set which temp sensor on the 1 Wire bus is the one for body readings and which is ambient
  bool WaitingForConversion=false;
  
  DeviceAddress SerialID[2];

  ReadBodySensorID(BodyTempSensor);
  sensors.begin();
  ClearScreen();
  tft.print(F("Release button."));
  // wait for button release
  while(digitalRead(3)==false);
  ClearScreen();
  tft.setTextColor(GREEN);
  tft.println("     Body Temp Sensor");
  tft.setTextColor(WHITE);
  tft.print(F("Place finger on body temp sensor\n\n\n\n\n\n\nWill store as  body sensorwhen 3C higher\nthan other sensor."));
  tft.setTextColor(YELLOW);
  if(Reboot)
    tft.println(F("\nReset MCU to exit"));
  else
    tft.println(F("\nPress button to exit"));
  tft.setCursor(2,52);
  tft.print(F("Temp  Allocation"));
  tft.setTextColor(WHITE);
  tft.drawRect(0,48,280,60,WHITE);
  tft.drawLine(68,48,68,107,WHITE);
  tft.drawLine(0,70,280,70,WHITE);
  sensors.setWaitForConversion(false);
  while(true)
  {
    if(digitalRead(3)==false)
    {
      delay(250); //debounce
      while(digitalRead(3)==false);
      delay(250); //debounce
      ClearScreen();
      return;
    }
    if(WaitingForConversion==false)
    {
      sensors.requestTemperatures(); // Send the command to get temperature 
      WaitingForConversion=true;
    }  
    if(sensors.isConversionComplete())
    {
      WaitingForConversion=false;
      for(uint8_t i=0;i<2;i++)
      {   
        if(sensors.getAddress(SerialID[i],i))
        {
          tft.setCursor(2,72+(i*18));
          tft.setTextColor(WHITE,BLACK);
          tft.print(sensors.getTempCByIndex(i));
          if(CompareSensorIDs(SerialID[i],BodyTempSensor))
          {
            tft.setTextColor(YELLOW);
            tft.print(F(" Body Temp Sensor"));
            tft.setTextColor(WHITE);
          }
        }
        else
        {
          tft.print(F("Temp Sensor "));
          tft.print(i);
          tft.print(F(" not found."));
        }
      }
    }
    // Check for one of the sensors being 3oC higher than the other
    if(sensors.getTempCByIndex(0)>sensors.getTempCByIndex(1)+2.99)
    {
      StoreBodySensor(0,BodyTempSensor);
      ReadBodySensorID(BodyTempSensor);
    }
    if(sensors.getTempCByIndex(1)>sensors.getTempCByIndex(0)+2.99)
    {
      StoreBodySensor(1,BodyTempSensor);
      ReadBodySensorID(BodyTempSensor);
    }
  }
}

void ReadBodySensorID(uint8_t *SerialID)
{  
  for(uint8_t i=0;i<8;i++)
    SerialID[i]=EEPROM.read(i);
}

void CopyTempSensorID(uint8_t *CopiedFrom,uint8_t *CopiedTo)
{  
  for(uint8_t i=0;i<8;i++)
    CopiedTo[i]=CopiedFrom[i];
}

void StoreBodySensor(uint8_t Idx,uint8_t *CurrentID)
{
  // store the unique address of the body sensor  
  DeviceAddress SerialID;
  sensors.getAddress(SerialID,Idx);
  if(CompareSensorIDs(SerialID,CurrentID)==false)
  {
    for(uint8_t i=0;i<8;i++)
      EEPROM.update(i,SerialID[i]);
  }
}

bool CompareSensorIDs(uint8_t *SerialID1,uint8_t *SerialID2)
{
  uint8_t i=0;

  while(i<8)
  {
    if(SerialID1[i]!=SerialID2[i])
      return false;
    i++;
  }
  return true;
}


void BootSensors()
{
  ClearScreen();
  if(QuickBoot==false)
  {
    tft.setTextColor(WHITE);  
    tft.setTextSize(1);
  
    // boot sequence, some of this is not for show, it checks the availability of the various sensors
    tft.println(F("Starfleet Sickbay Monitor Ver. 1.0"));
    delay(2000);
    tft.println(F("ID:USS Enterprise 1701 Bay 1\nSystem Checks..."));
    BootTemp();
    BootPox();  
    BootLung();
    tft.println("");
    tft.setTextColor(RED); 
    tft.print("Warning: ");
    tft.setTextColor(WHITE); 
    tft.println(F("Unsuitable for Vulcan physiology"));
    delay(1000);
    tft.print("\n\nStarting");
    for(uint8_t i=0;i<15;i++)
    {
      tft.print(".");
      delay(250);
    }
  }
  else
  {
    BootPox();
    BootTemp(); 
  }
}


void BootLung()
{
  if(QuickBoot==false)
  {
    tft.setTextColor(WHITE);
    tft.print("\nLung : ");
    delay(1000);
    if(PulseWorking==false)
    {    
      tft.setTextColor(RED);  
      tft.println(F("\nNEGATIVE\nData will be unavailable"));
    }
    else
    {    
      tft.setTextColor(GREEN); 
      tft.println(F("AFFIRMATIVE"));
    }  
  }
  else
  {
    if(PulseWorking==false)
    {      
      tft.println(F("No lung sensor if pulse not working."));
      delay(2000);
    }
  }
}


void BootPox()
{
  tft.setTextColor(WHITE);  
  if(QuickBoot==false)
  {    
    delay(2000);
    tft.print(F("\nOxygen and pulse sensor : "));
    delay(2000);
  }
  interrupts();         // Ensure interupts are enabled, otherwise Pox won't work
  if(!pox.begin()) 
  {
    PulseWorking=false;
    if(QuickBoot)    
    {         
      tft.println(F("Pulse Sensor not working"));
      delay(2000);
    }
    else
    {
      tft.setTextColor(RED);  
      tft.println(F("\nNEGATIVE\nData will be unavailable"));
    }
  } 
  else 
  {
    if(QuickBoot==false)
    {
      tft.setTextColor(GREEN); 
      tft.println(F("AFFIRMATIVE"));
    }
    pox.shutdown();
  }  
}

void BootTemp()
{   
  uint8_t SensorsFound=0;
  DeviceAddress TempAddress0,TempAddress1;  
  tft.setTextColor(WHITE); 
  if(QuickBoot==false)
  {
    tft.print(F("\nTemperature : "));
    delay(1000);  
  }  
  sensors.begin();
  sensors.setWaitForConversion(true);
  sensors.requestTemperatures(); // Send the command to get temperatures
  if(sensors.getTempCByIndex(0)!=-127)
    SensorsFound++;
  if(sensors.getTempCByIndex(1)!=-127)
    SensorsFound++;
  if(SensorsFound!=2)
  {    
    TemperatureWorking=false;
    if(QuickBoot)
    {      
      if(SensorsFound==1)
        tft.println(F("Missing 1 temperature sensor"));
      else
        tft.println(F("Missing 2 temperature sensors"));
      delay(2000);
    }
    else
    {
      tft.setTextColor(RED);  
      if(SensorsFound==1)
        tft.println(F("NEGATIVE on 1 temperature sensor."));
      else
        tft.println(F("NEGATIVE on 2 temperature sensors."));
      tft.println(F("Data will be unavailable."));
    }
  }
  if((SensorsFound==2)&(QuickBoot==false))
  {
    tft.setTextColor(GREEN); 
    tft.println(F("AFFIRMATIVE on both sensors."));
  }    
  // Need to check if one of the sensors has been assigned as the body sensor
  if(SensorsFound==2)
  {    
    ReadBodySensorID(BodyTempSensor);
    sensors.getAddress(TempAddress0,0);
    sensors.getAddress(TempAddress1,1);
    if((CompareSensorIDs(TempAddress0,BodyTempSensor)==false)&(CompareSensorIDs(TempAddress1,BodyTempSensor)==false))      // No sensor has been defined as the body sensor, prompt user
      SetUpTempSensors(true);    
    else
    {
      if(CompareSensorIDs(TempAddress0,BodyTempSensor))  // If this is the body sensor then sensor with index 1 is the environment
        CopyTempSensorID(TempAddress1,EnvironmentTempSensor);
      if(CompareSensorIDs(TempAddress1,BodyTempSensor))  // If this is the body sensor then sensor with index 0 is the environment
        CopyTempSensorID(TempAddress0,EnvironmentTempSensor);
    }
  }
}

void UpdateTemp()
{
  float EnviromentTemp;  // Temp of surroundings, having a seperate sensor measuring this as well as the skin
                         // temp gives a much better estimate of the body's internal core temperature.

  if(WaitingForTempReading==false)
  {
    sensors.setWaitForConversion(false);
    sensors.requestTemperatures(); 
    WaitingForTempReading=true; 
  }
  else
  {
    if(sensors.isConversionComplete())
    {
      TempReading=sensors.getTempC(BodyTempSensor);     // This is Skin Temperature reading
      EnviromentTemp=sensors.getTempC(EnvironmentTempSensor);
      // Convert (estimate) core temperature by using the work of Loughborough University (UK) as a base 
      // point to this calculate, slight alts for this temp sensor and envirnoment have been implemented
      TempReading=7.07+(1.065 *TempReading)-(0.087*EnviromentTemp);
      TempReading=(1.8 * TempReading)+32;         // convert to Fahrenheit
      WaitingForTempReading=false;
    }
  }

  
}


void UpdatePox()
{
  // takes several seconds to get a correct reading. Normally this just happens once at the start
  // of putting your finger on, but because we have to switch the sensor on and off (due to the
  // compatibility issues with the temperature library) then we need to ensure for each call to
  // this routine that we allow time (7 seconds) for the sensor to settle. 7 seconds was found
  // to be a reasonable comprimise.
  
  uint32_t Start = 0;

  interrupts();         // Ensure interupts are enabled, otherwise Pox won't work
  pox.resume();         // Tell to resume readings.
  // Keep telling the sensor to update (basically take readings I think) for 7 seconds
  for(uint8_t i=0;i<7;i++)
  {
    Start=millis();
    while(millis() - Start < 1000)
    {
      if(i>2)           // ignore first couple of values, they tend to be bogusly low
      {
        HeartReading=pox.getHeartRate();     // Now get the rate  
        BloodO2Reading=pox.getSpO2();
        if((HeartReading==0)|(BloodO2Reading==0))
          LungReading=0;
        else
          LungReading=(BloodO2Reading/HeartReading)*15;   // arbitary calculation
      }
      pox.update();
      PlotReadings();
    }
  }
  pox.shutdown();                   // shutdown pox (go into low power mode), 
                                    // not doing this and calling the temperature reading
                                    // will cause the Pox to not work
}

void UpdateHeartBeatSound()
{
  // plays the heart beat sound if required, looking at the current BPM.
  // Originally the sound played when a pulse was detected but the MAX30100 is a little
  // unpredictable over short periods of time for the pulse, least my set up was and it
  // would lead to irrugular sounding heart beats. so we play the sound an intervals
  // that corrospond to the current displayed BPM

  static uint32_t LastBeatTime=0;
  uint32_t CurrentTime;
  uint32_t TimeBetweenBeats;
  static bool SoundStarted=false;
  if(Sound)                           // only bother if sound option enabled
  {
    CurrentTime=millis(); 
    if(SoundStarted)                        // we've started a sound, we need to being the pin high
    {                                       // again or it will play continously. But not too quickly
      if(CurrentTime>LastBeatTime+125)      // else it won't play at all, 125ms seems to work well
      {
        digitalWrite(4,HIGH);
        SoundStarted=false;
      }
    }
    else
    {
      if(Heart.Reading>20)               // we have a pulse, always a good thing!
      {
        // Is it time for another beat?
        TimeBetweenBeats=(60/Heart.Reading)*1000;           // get time between beats in ms
        if(CurrentTime>LastBeatTime+TimeBetweenBeats)
        {
          digitalWrite(4,LOW);                              // time for another
          SoundStarted=true;
          LastBeatTime=CurrentTime;
          DrawPulseLight(true);
        }
      }
    }
  }
}


void PlotReadings()
{  
  
  PlotReading(&Temp,TempReading);
  PlotReading(&BloodO2,BloodO2Reading);
  PlotReading(&Heart,HeartReading);
  PlotReading(&Lungs,LungReading);
  
  // handle other essential tasks that must occur quite often
  UpdateHeartBeatSound();
  if(PulseStartTime!=0)
  {
    if(millis()>PulseStartTime+250)
      DrawPulseLight(false);
  }  
  if(RespireDelay!=0)
  {
    if(millis()>RespireDelayStartTime+RespireDelay)
    {
      RespireStartTime=millis();
      DrawRespireLight(true);
    }
  }
  else
  {
    if(RespireStartTime!=0)
    {
      if(millis()>RespireStartTime+250)
        DrawRespireLight(false);
    }
  }
}



void PlotReading(Gauge_Struct *Gauge,float CurrentReading)
{
  // This code moves the pointer a little bit at a time, so there are no rapid jumps to any location on the gauge
  // and things appear to move smoothly - as in the TV series.
  float AmountToMoveBy=Gauge->AmountToMoveBy;
  if(CurrentReading!=Gauge->Reading)
  {
    //decide how much should move by,
    if((Gauge->Reading>Gauge->AmberStart)&(Gauge->Reading<Gauge->AmberEnd))
      AmountToMoveBy/=2;
    if(CurrentReading<Gauge->StartValue)     // do not allow to go below start
      CurrentReading=Gauge->StartValue;
    if(CurrentReading>Gauge->Reading)  
    {    
      Gauge->Reading+=AmountToMoveBy;
      if(CurrentReading<Gauge->Reading) // If we've undershot then we have reached the temp
        Gauge->Reading=CurrentReading;
    }
    else       
    {
      Gauge->Reading-=AmountToMoveBy;
      if(CurrentReading>Gauge->Reading) // If we've overshot then we have reached the temp
        Gauge->Reading=CurrentReading;
    }      
    tft.drawBitmap(Gauge->XPos+Gauge->PointerOffset, ValueToScreenY(Gauge,Gauge->Reading)-GAUGE_LINE_WIDTH,  PointerGfx, 16, 11,WHITE,BLACK);
  }
}


void InitGauges()
{
  Temp.XPos=30;
  Temp.PointerOffset=26;
  strcpy(Temp.Label,"Temp");
  strcpy(Temp.SubLabel,"F  C");
  Temp.StartValue=78;
  Temp.MaxValue=112;
  Temp.IncrementValue=2;
  Temp.AmberStart=94.1;
  Temp.AmberEnd=102.2;
  Temp.GreenStart=96.5;
  Temp.GreenEnd=100.3;
  Temp.DrawFullHorizontalLine=false;
  Temp.Divisions=18;
  Temp.Reading=98;
  Temp.AmountToMoveBy=0.1;
  
  BloodO2.XPos=285;  
  BloodO2.PointerOffset=8;
  strcpy(BloodO2.Label,"Blood");
  strcpy(BloodO2.SubLabel,"% O2");
  BloodO2.StartValue=80;
  BloodO2.MaxValue=100;
  BloodO2.IncrementValue=2;
  BloodO2.AmberStart=90;
  BloodO2.AmberEnd=100;
  BloodO2.GreenStart=95;
  BloodO2.GreenEnd=100;
  BloodO2.DrawFullHorizontalLine=true;
  BloodO2.Divisions=14;
  BloodO2.Reading=96;
  BloodO2.AmountToMoveBy=0.1;
  
  Heart.XPos=220;
  Heart.PointerOffset=8;
  strcpy(Heart.Label,"Heart");
  strcpy(Heart.SubLabel,"BPM");
  Heart.StartValue=20;
  Heart.MaxValue=152;
  Heart.IncrementValue=10;
  Heart.AmberStart=40;
  Heart.AmberEnd=100;
  Heart.GreenStart=50;
  Heart.GreenEnd=95;
  Heart.DrawFullHorizontalLine=true;
  Heart.Divisions=14;
  Heart.Reading=65;
  Heart.AmountToMoveBy=0.3;
  
  Lungs.XPos=100;
  Lungs.PointerOffset=8;
  strcpy(Lungs.Label,"Lungs");
  strcpy(Lungs.SubLabel,"");
  Lungs.StartValue=0;
  Lungs.MaxValue=27;
  Lungs.IncrementValue=2;
  Lungs.AmberStart=9;
  Lungs.AmberEnd=26;
  Lungs.GreenStart=11.5;
  Lungs.GreenEnd=26;
  Lungs.DrawFullHorizontalLine=true;
  Lungs.Divisions=14;
  Lungs.Reading=(BloodO2.Reading/Heart.Reading)*9.5;
  Lungs.AmountToMoveBy=0.1;
}


void DrawPanel()
{
  // draws the main parts of the panel that do not animate
  ClearScreen();
  tft.setTextColor(WHITE);  
  tft.setTextSize(1);
  DrawGauge(&Temp);
  DrawGauge(&BloodO2);
  DrawGauge(&Heart);
  DrawGauge(&Lungs);
  tft.setCursor(124, 50);
  tft.print(F("RESPIRATION"));
  tft.setCursor(141, 105);
  tft.print(F("PULSE"));
  tft.setCursor(138, 122);
  tft.print(F("ADJUST"));
  tft.setCursor(126, 131);
  tft.print(F("FOR NORMAL"));
  tft.setCursor(132, 160);
  tft.print(F("RECORDER"));
  tft.fillRoundRect(137, 170, 16, 8,2, GREEN);
  tft.drawRoundRect(159, 170, 16, 9,2, RED); 
  DrawRespireLight(false);
  DrawPulseLight(false);
  // Add in Celcius scale, 
  float YStart=SCALE_Y+SCALE_HEIGHT-18;
  for(uint16_t i=27;i<45;i+=2)
  {    
    tft.fillRect(36, YStart, 4, 3, RED);  
    tft.setCursor(43, YStart-2);
    tft.print(i);
    YStart-=19.5;
  }
}


void DrawPulseLight(bool On)
{
  uint16_t Colour;
  
  if(On)
  {
    Colour=RED;
    PulseStartTime=millis();
    if((RespireDelay==0)&(RespireStartTime==0))
    {
      RespireDelay=random(250,2000);
      RespireDelayStartTime=PulseStartTime;
    }
  }
  else
  {
    Colour=DARK_RED;
    PulseStartTime=0;
  }
  tft.fillCircle(155,85,15,Colour);
}


void DrawRespireLight(bool On)
{
  uint16_t Colour;
  
  if(On)
  {
    Colour=RED;
    RespireDelayStartTime=0;
    RespireDelay=0;
  }
  else
  {
    Colour=DARK_RED;
    RespireStartTime=0;
  }
  tft.fillCircle(155,30,15,Colour);
}


uint16_t ValueToScreenY(Gauge_Struct *Gauge,float Value)
{
  // Given the Value returns the Screen Y Position for this point on the screen for this graph
  float MarkerInterval;
  float Temp;
  
  MarkerInterval=float(SCALE_HEIGHT)/float(Gauge->Divisions);       // Distance between gauge lines for this gauge
  Temp=(float(Value-Gauge->StartValue)/Gauge->IncrementValue)*MarkerInterval;
  return (SCALE_Y+(SCALE_HEIGHT-Temp)-GAUGE_LINE_WIDTH);
}


void DrawGauge(Gauge_Struct *Gauge)
{
  float GaugeLineYPos;
  float MarkerInterval;
  char OutString[4];
  uint32_t ScaleLineColour;
  uint8_t GaugeLineLength;
  float LineYStart,LineHeight;       // for amber and red parts of the gauge 
  uint16_t CurrentGaugeValue;

  MarkerInterval=float(SCALE_HEIGHT)/float(Gauge->Divisions);       // Distance between gauge lines for this gauge
  
  tft.fillRoundRect(Gauge->XPos, SCALE_Y, SCALE_WIDTH, SCALE_HEIGHT, 1, RED);       // Main red bit
  // Amber/yellow bit
  LineYStart=ValueToScreenY(Gauge,Gauge->AmberEnd);
  LineHeight=ValueToScreenY(Gauge,Gauge->AmberStart);
  LineHeight=LineHeight-LineYStart;
  tft.fillRect(Gauge->XPos, LineYStart, SCALE_WIDTH, LineHeight, YELLOW);  // Add on Amber range
  
  // green bit
  LineYStart=ValueToScreenY(Gauge,Gauge->GreenEnd);
  LineHeight=ValueToScreenY(Gauge,Gauge->GreenStart);
  LineHeight=LineHeight-LineYStart;
  tft.fillRect(Gauge->XPos, LineYStart, SCALE_WIDTH, LineHeight, GREEN);  // Add on Green range
  
  tft.setTextColor(YELLOW);   
  tft.setCursor((Gauge->XPos+(SCALE_WIDTH/2))-((strlen(Gauge->Label)*6)/2), SCALE_Y+SCALE_HEIGHT+6);
  tft.print(Gauge->Label);
  tft.setCursor((Gauge->XPos+(SCALE_WIDTH/2))-((strlen(Gauge->SubLabel)*6)/2), SCALE_Y+SCALE_HEIGHT+16);
  tft.print(Gauge->SubLabel);

  if(Gauge->DrawFullHorizontalLine)
    GaugeLineLength=12;
  else
    GaugeLineLength=4;
  GaugeLineYPos=SCALE_Y+SCALE_HEIGHT-GAUGE_LINE_WIDTH;
  CurrentGaugeValue=Gauge->StartValue;
  while(GaugeLineYPos>=SCALE_Y)
  {
    ScaleLineColour=RED;
    if((CurrentGaugeValue>=Gauge->AmberStart)&(CurrentGaugeValue<=Gauge->AmberEnd))
    {
      if((CurrentGaugeValue>=Gauge->GreenStart)&(CurrentGaugeValue<=Gauge->GreenEnd))
        ScaleLineColour=GREEN;      
      else
        ScaleLineColour=YELLOW;      
    }
    tft.fillRoundRect(Gauge->XPos-3, GaugeLineYPos, GaugeLineLength, GAUGE_LINE_WIDTH, 1,ScaleLineColour);  
    itoa(CurrentGaugeValue,OutString,DEC);
    if(CurrentGaugeValue % 10==0)
    {
      if(CurrentGaugeValue>=100)
        tft.setCursor(Gauge->XPos-22, GaugeLineYPos-3);
      else
        if(CurrentGaugeValue>=10)
           tft.setCursor(Gauge->XPos-16, GaugeLineYPos-3);
         else
           tft.setCursor(Gauge->XPos-10, GaugeLineYPos-3);
      tft.print(OutString);
    }
    else
    {
      tft.setCursor(Gauge->XPos-10, GaugeLineYPos-3);
      if(CurrentGaugeValue>=100)
        tft.print(OutString[2]);
      else
        if(CurrentGaugeValue>=10)
          tft.print(OutString[1]);
        else        
          tft.print(OutString[0]);
    }
    GaugeLineYPos-=MarkerInterval;
    CurrentGaugeValue+=Gauge->IncrementValue;
  }
}
